/* FILE: diag.c                 Copyright (c), Symmetric Research, 1992-2000
 *
 * Run memory diagnostics on the DSP32C coprocessor board.
 *
 *
 *
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>

#ifndef SROS_LINUX
#include <conio.h>
#endif

#include "dspio.h"





/* INCLUDE THE HEXIFIED DIAG.DSP AND ITS SYMBOL TABLE DIAG.DSC:
 *
 *
 *
 *
 */

#include "diag.dsh"
#include "diag.dsc"





/* COMMAND LINE USAGE AND ERRORS:
 *
 *
 *
 */

char *DOS_UsageMsg[] = {
                
   "DSP32C diag : (c) SR 1999 : Rev {01/25/00}                          \n",
   "                                                                    \n",
   "USAGE: diag [/n] {addr:nbytes:waits} [addr:nbytes:waits]            \n",
   "                                                                    \n",
   "   /n = cpu to test on DSPMUL boards (0,1,2,3)                      \n",
   "                                                                    \n",
   "     and {addr:nbytes:waits} is a memory area to test, where:       \n",
   "                                                                    \n",
   "           addr = hex starting address                              \n",
   "         nbytes = hex number of bytes to test (<= 7FFFFF)           \n",
   "          waits = wait states (0,1,2,3)                             \n",
   "                                                                    \n",
   "        example: diag 600000:100000:0   < test mbyte 6 at 0 ws      \n",

   NULL,

  };

char *ERROR_DeviceDriver[] = {
        
   "Unable to open device driver ...                                    \n",
   "                                                                    \n",
   "   Check that the NT or Linux device driver is installed and started\n",
   "                                                                    \n",

   NULL,

  };

char *ERROR_InitDSP32C[] = {
        
   "Unable to initialize board ...                             \n",
   "                                                           \n",
   "   Check that                                              \n",
   "     1. DSP32C coprocessor board is installed ...          \n",
   "     2. DSP32C base IO address switch is either:           \n",
   "                                                           \n",
   "       * set to the 0x280 default or                       \n",
   "       * agrees with the DSP32C_BASEIO environment variable\n",
   "                                                           \n",
   "   see the Users manuals for information on switch settings\n",
   "                                                           \n",

   NULL,

  };


void DOS_Usage( void) {
        
        int i;


        /* Print out the DOS command line usage message and quit. */

        for ( i = 0 ; DOS_UsageMsg[i] ; i++ )

                printf( DOS_UsageMsg[i]);
        
        exit( 1);

}       
void DOS_Error( char *msg) {


        /* Print out a DOS command line error and quit. */

        printf( "ERROR: %s", msg);
        
        exit( 1);

}
void DOS_ErrorManyLine( char *msg[]) {

        int i;

        /* Print out a multi-line error and quit. */

        for ( i = 0 ; msg[i] ; i++ )

                printf( "%s", msg[i]);
        
        exit( 1);

}



/* Keyboard helper function */

int stickykey = EOF;

int GetStickyKey( void ) {

        int c;

#ifdef SROS_LINUX
        c = getchar();
#else
        c = EOF;
	if ( kbhit() ) {
	        c = getch();
	        if (c == 0xD) printf("\n"); /* add newline if char was ENTER */
                }
#endif

	/* Save the first keystroke, then return it thereafter (until reset) */

	if (stickykey == EOF)
                stickykey = c;
        else
                c = stickykey;

	return( c );
}




/* DSP32C PCW WAIT STATES ARRAY:
 *
 *
 *
 *
 */

unsigned

        pcw[] = {       /* pcw register wait state values */
                
                0x0000 | 0x0000,        /* 0 waits for A, 0 waits for B */
                0x0002 | 0x0001,        /* 1 waits for A, 1 waits for B */
                0x0012 | 0x0005,        /* 2 waits for A, 2 waits for B */ 
                0x0022 | 0x0009,        /* 3 waits for A, 3 waits for B */
                        
                };





/* DSP32C MEMORY MODE AND ADDRESSES:
 *
 * * ONLY MEMORY MODE 5 USED
 *
 * The jumpers on the DSP32C board are wired to use memory mode 5.  The
 * internal ram addresses below are appropriate for this mode.
 *
 *
 */

#define MEMORY_MODE     5
#define ULONG           unsigned long

ULONG RAM[3] = {      /* internal ram addresses */
        
        0x000800L,              /* block 0 */
        0xFFF800L,              /* block 1 */
        0x000000L,              /* block 2 */

        };





/* DSP32C MEMORY AREAS TO TEST:
 *
 * These areas are specified on the DOS command line.
 *
 *
 *
 *
 */

#define MAXAREAS        4

unsigned nareas;

struct memory_area {

        ULONG address;
        ULONG nbytes;
        unsigned waits;

        } memory[MAXAREAS];







/* DSP32C DIAGNOSTIC FUNCTIONS:
 *
 *
 *
 *
 */

int count( ULONG address, unsigned nbytes, ULONG *error ) {

        unsigned short i, temp;


        /* Run a simple counting test on the DSP32C from the PC. */

        for ( i = 0 ; i < nbytes ; i += 2 )

                DSP32C_Put( &i, address+i, sizeof( unsigned short));



        /* Read the block back starting at the beginning. */

        for ( i = 0 ; i < nbytes ; i += 2 ) {
                
                DSP32C_Get( address+i, &temp, sizeof( unsigned short));

                if ( temp != i )
                        
                        { *error = address+i; return( 0); /* failure */ }
                }



        return( 1);

}
int execute( struct memory_area *m, ULONG *error, ULONG DSP_func) {

        struct input_array { ULONG status, address, nbytes, waits; } input;

            ULONG inst, nop;



        /* Run a DSP32C diagnostic. */

        input.status   = DSP_COMPUTING;

        input.address  = m->address;
        input.nbytes   = m->nbytes;
        input.waits    = pcw[m->waits];

          DSP32C_Put( &input, DSP_input_array, sizeof( input));



        /* Poke the DSP32C reset vector with a goto and nop; */

#define GOTO    0xA0000000L
#define NOP     0x00000000L

        inst = GOTO | ((DSP_func&0x00FF0000L)<<5) | (DSP_func&0xFFFFL);
         nop = NOP;

        DSP32C_Put( &inst, 0x0L, sizeof( ULONG));
        DSP32C_Put(  &nop, 0x4L, sizeof( ULONG));
        


        /* Execute, checking the status flag to see when done. */
            
        DSP32C_Go();
        
            while ( input.status == DSP_COMPUTING ) {
                        

                /* Poll the process. */

                DSP32C_Get( DSP_input_array, &input, sizeof( input));


                /* Let the user break out with a keyboard hit. */

                if ( GetStickyKey() != EOF )

                        { input.status = DSP_SUCCESS; break; }

                }

        DSP32C_Halt();


        /* Return success or failure. */

        if ( input.status == DSP_SUCCESS )
                
              return( 1);
        
        else
                { *error = input.address; return( 0); }

}







/* RUN A PARTICULAR DSP32C DIAGNOSTIC:
 *
 *
 *
 *
 */

void InternalRamTest( void) {
        
        unsigned i, nbytes; ULONG address, error;


        /* Test the internal RAM blocks from the PC side. */
        
        printf( "\tTesting internal ram in memory mode %d\n\n", MEMORY_MODE);

        for ( i = 0 ; i < 3 ; i++ ) {

                address = RAM[i];
                 nbytes = 0x800;

                printf( "\t\tRAM%d at 0x%06lX, 0x%03X bytes ... ",

                                                      i, address, nbytes);


                if ( count( address, nbytes, &error) )

                        printf( "good\n");


                else {
                        printf( "error at 0x%06lX\n", error&0xFFFFFF);
                        exit( 1);

                        }
                }


        printf( "\n");


}
void test( char *message, ULONG DSP_func ) {

        unsigned area; ULONG pass, error;
        

	/* Reset sticky key for this test */

        stickykey = EOF;


        /* Run a DSP32C diagnostic on all the memory areas. */

        printf( "\t%s ... ENTER to continue\n", message);

        area = 0; pass = 0;

        while ( 1 ) {

               /* Run the diagnostic. */

               printf( "\r\t\tarea = %d   pass = 0x%08lX  ... ", area, pass);

                  if ( !execute( &memory[area], &error, DSP_func) ) {
                        
                        printf( "error at 0x%06lX\n", error&0xFFFFFF);

                            exit( 1);
                        }

                /* Increment to the next memory area. */
                     
                if ( ++area == nareas )

                        { area = 0; pass++; }


		/* Any key pressed in the execute call will stick */

                if ( GetStickyKey() != EOF )
                        break;

                }
                


        printf( "\r\t\t                                  good\n\n");

}







/* MAIN:
 *
 * Process the DOS command line and run the diagnostics.
 *
 *
 *
 */

int main( int argc, char **argv) {
        
        int ncpu;

            int area; ULONG a, n; unsigned w;



        /* Make sure some command line arguments are specified. */
            
        argc--; argv++; /* skip argv[0] */

          if ( argc < 1 )

                  DOS_Usage();


#ifdef SROS_LINUX

        /* Set Linux terminal to allow for empty keyboard checks */

          if ( SetNonBlock( 1 ) == -1 )
                 { printf("Can't set non-blocking read\n");  exit( 1 ); }
#endif




        /* Get the DOS command line arguments. */

        for ( ncpu = -1 , area = 0 ; argc-- ; argv++ ) {
                        

                /* Check for a DSPMUL cpu to test. */
                
                if ( sscanf( *argv, "/%1d", &ncpu) == 1 ) {
                        
                        if ( ncpu < 0 || 3 < ncpu )
                                
                            DOS_Error( "For the /n cpu option, n = 0,1,2,3");

                        }


                /* Check for a memory area to test. */

                else if ( sscanf( *argv, "%lx:%lx:%d", &a, &n, &w) == 3 ) {


                        if ( area >= MAXAREAS )
                                
                                DOS_Error( "Too many memory areas");


                        memory[area].address = a;
                        memory[area].nbytes  = n;
                        memory[area].waits   = w;

                          area++;


                        if ( a > 0xFFFFFFL )
                                
                                DOS_Error( "addr must be <= 0xFFFFFF");


                        if ( n > 0x7FFFFFL )
                                
                                DOS_Error( "nbytes must be <= 0x7FFFFF");


                        if ( w > 3 )
                                
                                DOS_Error( "waits must be 0,1,2, or 3");
                             

                        }


                /* Otherwise an unrecognized command line option. */

                else

                      DOS_Usage();

                }



        /* Save the number of specified memory areas to test. */

        nareas = area;




        /* Make sure at least one memory area is specified. */
        
        if ( nareas == 0 )
                
                DOS_Error( "At least one memory area must be specified");





        /* Select the cpu to test for DSPMUL boards. */
          
        printf( "\n");

        if ( ncpu >= 0 ) {
                
                printf( "    Selecting DSPMUL cpu %1d\n\n", ncpu);

                DSP32C_SelectCpu( ncpu);

                }




        /* Initialize the device driver (if applicable) and the DSP32C. */

        if ( !DSP32C_Open() )

                DOS_ErrorManyLine( ERROR_DeviceDriver);

        if ( !DSP32C_Init() )

                DOS_ErrorManyLine( ERROR_InitDSP32C);




        /* Report the base address. */

        printf( "\tDSP32C found at PC IO address = 0x%X\n\n", DSP32C_BaseIo);




        /* Test the internal RAM blocks from the PC side. */
     
        InternalRamTest();




        /* Download the DSP32C portion of the diagnostics. */

        DSP32C_SrHexLoad( diag_dsh);




        /* Run the slamming ones test. */
        
        test( "Slamming ones", DSP_slam);


        /* Run the sliding ones test. */

        test( "Sliding ones", DSP_slide);




        /* Success, return to DOS. */
        
        exit( 0);
}
