/* FILE: quick.dsp              Copyright (c), Symmetric Research, 1993 1998
 *
 * This program demonstrates quick external interrupts on the DSP32C.
 *
 * It is essentially the same as intrupt.dsp, except that the interrupt
 * service routine is programmed directly in the IVTP table.
 *
 * This saves the overhead of making a branch to a separate isr as in
 * intrupt.dsp, but of course only a single instruction followed by an
 * ireturn can normally be used.
 *
 * Note that if INTREQ1 is the only interrupt you are using, the number
 * of instructions in the IVTP table can be expanded, right over the other
 * interrupt vectors. 
 *
 * The two instruction quick interrupt scheme can be useful for reading
 * data from the parallel and serial ports.  As noted in the block
 * memory move example in the User's Guide, 32 bit parallel port users
 * may want to use instructions like a0 = (*r1++ = *r2) + a0 to move
 * data directly to memory.
 *
 *
 *
 */




/* DSP32C PCW interrupt mask bits:
 *
 * These constants define the particular bits that must be set in the
 * DSP32C's PCW register to enable a particular interrupt.
 *
 * Be careful not to corrupt the PCW's other bits when changing values.
 *
 *
 */

constant

	ENABLE_INTREQ1		= 0x8000,
	ENABLE_PDF		= 0x4000,
	ENABLE_PDE		= 0x2000,
	ENABLE_IBF		= 0x1000,
	ENABLE_OBE		= 0x0800,
	ENABLE_INTREQ2		= 0x0400;






/* Main program:
 *
 * Set up and wait for interrupts.
 *
 *
 */

main {

	register

		value	= r1,
		c	= r2,

		ivtp	= r22;  /* DSP32C dedicated ivtp pointer */



	/* Program the PCW register for zero wait states. */

	value = 0; pcw = value;



	/* Point ivtp to the interrupt vector table. */

	ivtp = IVTP_TABLE;



	/* Enable INTREQ1 and wait for interrupts. */

	value = pcw;

	    value |= ENABLE_INTREQ1; pcw = value;



	c = 0;

	wait:

		    /* Put any foreground code in here ... */

		*count = c;

		goto wait;


}

long count = 0;







/* IVTP interrupt vector table:
 *
 * In the quick interrupt method, the interrupt service routine instuctions
 * are included directly in the ivtp table instead of branching to the isr.
 *
 *
 *
 */

IVTP_TABLE {

	register c = r2;

    nops_off;

	c++;			     	/* interrupt vector #1 */
	ireturn;

	goto isr_PDF;			/* interrupt vector #2 */
	nop;

	goto isr_PDE;			/* interrupt vector #3 */
	nop;

	goto isr_IBF;			/* interrupt vector #4 */
	nop;

	goto isr_OBE;			/* interrupt vector #5 */
	nop;

	goto isr_INTREQ2;		/* interrupt vector #6 */
	nop;

    nops_on;

}








/* Interrupt service routines:
 *
 * Stubbs for unused isr routines ...
 *
 *
 *
 *
 */

isr_INTREQ1 {

	ireturn;

}
isr_PDF {

	ireturn;

}
isr_PDE {

	ireturn;

}
isr_IBF {

	ireturn;

}
isr_OBE {

	ireturn;

}
isr_INTREQ2 {

	ireturn;

}
