/* FILE: sysdep.h
 *
 * This file has the prototypes for the system dependent function wrappers.
 * The top part of this file is common to all SR products.
 *
 * The bottom part has been customized for the DSP32C.
 *
 */


#ifndef _SYSDEP_H_
#define _SYSDEP_H_


/* Defines */

#define CHECK_READ  0
#define CHECK_WRITE 1

typedef void (*PKFN_TIMER)( unsigned long timer_arg );





/* System dependent functions called from SR device driver routines */

int  Linux_check_region(   unsigned long from, unsigned long extent );
void Linux_request_region( unsigned long from, unsigned long extent, 
                                                        const char* name );
void Linux_release_region( unsigned long from, unsigned long extent );

int  Linux_register_chrdev(   unsigned int major, const char* name );
int  Linux_unregister_chrdev( unsigned int major, const char* name );

int  Linux_access_ok( int type, const void* arg, unsigned long size );
void Linux_get_user_uchar( unsigned char* value, unsigned char* iobuffer );
void Linux_get_user_int(             int* value,           int* iobuffer );
void Linux_put_user_uchar( unsigned char  value, unsigned char* iobuffer );
void Linux_put_user_int(             int  value,           int* iobuffer );

unsigned long Linux_copy_from_user( void* tokernel, const void* fromuser,
                                                         unsigned long size );
unsigned long Linux_copy_to_user(   void* touser,   const void* fromkernel, 
                                                         unsigned long size);
unsigned long Linux_jiffies( void );

int  Linux_request_irq( unsigned int irq,
	 	        void (*handler)(int, void *, struct pt_regs *),
		        unsigned long flags, const char *device, void *dev_id);
void Linux_free_irq( unsigned int irq, void *dev_id );

void  Linux_init_timer( void *pktimer, PKFN_TIMER function,unsigned long data);
void  Linux_add_timer(  void *pktimer );
int   Linux_del_timer(  void *pktimer );
void* Linux_alloc_timer( void );
void  Linux_set_timer_expires( void *pktimer, unsigned long period );

void Linux_export_no_symbols( void );
void Linux_mod_inc_use_count( void );
void Linux_mod_dec_use_count( void );




/****************************************************************************/

/* DSP32C specific portion */


/* Shared global variables */

extern int port_baseio;
extern int port_basesel;
extern int port_count;
extern int dev_major;



/* Dsp32c module and dispatch functions called from sysdep routines */

int  dsp32c_init_module(     void );
void dsp32c_cleanup_module(  void );

int  dsp32c_open(  void **private_data );
int  dsp32c_close( void  *private_data );
int  dsp32c_ioctl( void  *private_data, unsigned int cmd, unsigned long arg);
unsigned long 
     dsp32c_read(  void  *private_data, char *buf, unsigned long count );

#endif /* _SYSDEP_H_ */



