/* FILE: dspmon.c               Copyright (c), Symmetric Research, 1998-2000
 *
 * DSP32C monitor/debugger program.  This is the Linux console only version.
 *
 *
 */

#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "dsptools.h"




/* Typedefs */

typedef struct {

        unsigned long value;
                 char *name;
                   int type;
        }

        SYMBOL;




/* Prototypes */

void  LL_PutC( int c);
void  LL_PutS( char *s, int newline);
int   LL_KeyIn( void);
int   LL_KeyPress( void);

int   StrEqu( char *src, char *dest);
int   StrScan( char *src, char *dest);
int   StrGet( char *dest, int maxchar);

void  DOS_Error( char *msg);
void  DOS_Warning( char *msg);
void  ShowErrMsg( char *msg, int is_error);

void    SortSymbols( void);
SYMBOL *SymbolicValue( char *name, int *type);
char   *SymbolicName( unsigned long value, int type);
int     ReadSymbols( char *filename);
void    ClearSymbolTable( void);

void  SetBreakpoint( void);
void  RestartFromBreakpoint( void);
int   WaitForBreak( void);
int   ExecuteBreakpoint( void);

void  ShowHelp( void);
void  ShowSymbols( void);

char *dmprintf( int type, unsigned long addr);
void  Memory( void);

char *uprintf( unsigned long addr);
void  Unassemble( void);

char *rprintf( int ifmt);
void  Registers( void);

void  Enter( void);
void  Clear( void);
void  Load( char *filename);

void  update_breakpoint( void);
void  kb_interrupt( void);

void  CommandGo( void);
void  CommandBreakpoint( unsigned long newbreak);
void  CommandStep( void);
void  CommandHalt( void);
void  CommandEnableEni( void);
void  CommandQuit( void);

int   space( void);
int   type( int *flag);
int   wtype( int *flag);
int   hexnum( int ndigits, unsigned long *value);
int   decnum( int ndigits, unsigned long *value);
int   address( unsigned long *value);
int   nbytes( unsigned long *value);
int   ndecimal( unsigned long *value);
char *filename( void);
int   eob( void);
int   eoc( void);

void CmdLineClear( void);
void CmdLineGets( char *msg);
void CmdLineProcessor( void);

int main( int argc, char **argv);



#define ShowError(msg) ShowErrMsg(msg,1)
#define ShowMsg(msg)   ShowErrMsg(msg,0)


#define MAXADDRESS      0x00FFFFFFL
#define SCREEN_WIDTH    80










/* LOW LEVEL SCREEN AND KEYBOARD SUPPORT:
 *
 *
 */

void LL_PutC( int c) {


        /* Display the character c. */

        printf( "%c", c );
        fflush( NULL );

}
void LL_PutS( char *s, int newline) {


        /* Display the string s at (i,j). */

        if (newline)
                printf( "%s\n", s );
        else
                printf( "%s", s );
        fflush( NULL );

}
int LL_KeyIn( void) {

        int c,cr;


        /* Wait for a keypress. */

        while ( (c = getchar()) == EOF ) { ; }



        /* Discard rest of line. */

        cr = c;
        while ( cr != '\n' )
                cr = getchar();


        /* Return the keycode. */

        return( c);


}
int LL_KeyPress( void) {
        

        /* Return the keypress status, but don't wait. */

        if ( getchar() != EOF )
                return( 1);
        else
                return( 0);

}
















/* STRING HELPERS:
 *
 * Generic string helper functions.
 *
 *
 *
 *
 */

int StrEqu( char *src, char *dest) {


        /* Compare the src and dest for exact equality. */

        while ( *src && *dest && *src == *dest )

                { src++; dest++; }


        return( *src == '\0' && *dest == '\0' ? 1 : 0 );


}
int StrScan( char *src, char *dest) {


        /* Scan src to see if it contains all of dest. */

        while ( *dest && *src == *dest )

                { src++; dest++; }

        return( *dest == '\0' ? 1 : 0 );


}
int StrGet( char *dest, int maxchar) {

        int   c, i, n;
        char *d;



        i = 0;
        n = maxchar - 1;
        d = dest;


        /* Wait for first character */

        while ( (c = getchar()) == EOF ) { ; }


        /* Get dest one character at a time until newline or maxchar. */

        while ( (c != '\n') && (i < n) ) {

                *d++ = c;

                i++;

                c = getchar();

                }

        *d = '\0';


        return( i );


}












/* DOS COMMAND LINE USAGE:
 *
 *
 *
 *
 *
 */

char *DOS_UsageMsg[] = {

    "                                                                   \n",
    " dspmon [/n] [file.dso [file.dss]] [file.out]                      \n",
    "                                                                   \n",
    "      /n             cpu n, where n = 0,1,2,3  (DSP_MUL only)      \n",
    "                                                                   \n",
    "      file.dso       SR .dso object file to download               \n",
    "      file.dss       SR .dss symbol table file to use              \n",
    "      file.out       WE .out fully located coff file to download   \n",
    "                                                                   \n",
    "      where [] denotes an optional item                            \n",

    NULL,

    };


void DOS_Error( char *msg) {

        int i;



        /* Give the user an error message and quit. */

        printf( "\nERROR: %s\n", msg);

        for ( i = 0 ; DOS_UsageMsg[i] ; i++ )

                printf( DOS_UsageMsg[i]);


        exit( 1);


}
void DOS_Warning( char *msg) {


        /* Give the user a warning. */

        printf( "\nWARNING: %s, ENTER to continue ... ", msg);
        fflush( NULL );

        LL_KeyIn();

        printf( "\n");

}




/* ERROR MESSAGE:
 *
 *
 *
 *
 */


void ShowErrMsg( char *msg, int is_error) {

        char buffer[SCREEN_WIDTH];


        if (is_error == 1)
                sprintf( buffer, "    Error: %s ...", msg);
        else
                sprintf( buffer, "    %s ...", msg);



        /* Show the message window. */

        LL_PutS( "", 1);
        LL_PutS( buffer, 1);
        LL_PutS( "", 1);

}









/* DSPASM SYMBOL TABLE AND LOOKUP FUNCTIONS:
 *
 * These functions read in and manage a DSPASM .dss symbol table for DSPMON.
 *
 * NOTE: The defined constants DSPASM_XXXX *MUST* agree with DSPASM.
 *
 *
 *
 */

#define DSPASM_CONS             0x0001
#define DSPASM_BYTE             0x0002
#define DSPASM_WORD             0x0004
#define DSPASM_LONG             0x0008
#define DSPASM_DSPF             0x0010
#define DSPASM_IEEE             0x0020
#define DSPASM_FUNC             0x0040
#define DSPASM_GOTO             0x0080

#define DSPASM_DATA             0x003E      /* = BYTE|WORD|LONG|DSPF|IEEE  */
#define DSPASM_PROGRAM          0x00C0      /* = FUNC|GOTO                 */
#define DSPASM_ALL              0x00FF      /* = CONS|DATA|PROGRAM         */





#define MAXSYMBOLS      300

SYMBOL symbol_table[ MAXSYMBOLS];

  unsigned nsymbols = 0;   /* actual number of symbols in table */




void SortSymbols( void) {

        int i, j; SYMBOL *s1, *s2, save;


        /* Sort the symbol table into value order. */

        for ( i = 1 ; i < nsymbols ; i++ ) {

                j = i-1;

                s1 = &symbol_table[i];
                s2 = &symbol_table[j];

                save = *s1;

                while ( j-- >= 0 && save.value < s2->value )

                        *s1-- = *s2--;

                *s1 = save;

                }


}
SYMBOL *SymbolicValue( char *name, int *type) {

        unsigned i; SYMBOL *s;



        /* Do a linear search on the symbol table for the specified name. */

        for ( i = 0 , s = symbol_table ; i < nsymbols ; i++ , s++ ) {

                if ( StrEqu( name, s->name) )

                        { *type = s->type; return( s); }
                }

        return( NULL);



}
char *SymbolicName( unsigned long value, int type) {

        unsigned imin, imax, itest;

                unsigned long min, max, test;


        /* Binary search for a symbol of the specified value and type. */

        imin = 0;
        imax = nsymbols - 1;

          min = symbol_table[imin].value;
          max = symbol_table[imax].value;


        if ( nsymbols > 0 && min <= value && value <= max ) {

                while ( imin <= imax ) {

                        itest = (imax + imin)/2;

                          test = symbol_table[itest].value;


                        if ( value > test )

                                imin = itest+1;

                        else if ( value < test )

                                imax = itest-1;

                        else {

                             if ( symbol_table[itest].type & type )

                                      return( symbol_table[itest].name);

                                else
                                        return( "");

                                }
                        }
                }


        /* Failed return. */

        return( "");


}
int ReadSymbols( char *filename) {

        FILE *input;

          SYMBOL *s;

              char buffer[80], name[129]; unsigned long value;

                      int n, type;



        /* Open and read a DSPASM .dss ASCII symbol table file. */

        if ( !(input = fopen( filename, "r")) )

                return( 0);



        /* Check for the proper input file type. */

        if ( fscanf( input, "Global symbol values from %s", buffer) != 1 )

                return( 0);




        /* Read in the symbols. */

               s = symbol_table;
        nsymbols = 0;


read:

      n = fscanf( input, "%128s %lx %s (%x)\n", name, &value, buffer, &type);

          if ( n == 4 ) {

                s->name  = strdup( name);
                s->value = value;
                s->type  = type;

                  if ( s->name && nsymbols < MAXSYMBOLS )

                          { s++; nsymbols++; goto read; }
                }





        /* Sort the table into value order. */

        SortSymbols();




        /* Check to see if the entire file was read or not. */

        if ( feof( input) )

                return( 1);


        else
                /* return success if at least some symbols were read ... */

                return( nsymbols > 0 ? 1 : 0 );



}
void ClearSymbolTable( void) {

        /* Clear the symbol table. */

        nsymbols = 0;

}









/* DSP32C EXECUTION AND BREAKPOINT CONTROL:
 *
 * The following is for controlling the execution of the DSP32C.  Both
 * the g and s DSPMON commands use this code.  Note that is the DSP32C
 * never reaches a breakpoint, then any PC keyboard hit will interrupt
 * the execution and return control to the monitor.
 *
 *
 * NOTES:
 *
 * * ALL BREAKPOINTING IS DONE IN SOFTWARE
 *
 * The DSP32C does not have hardware breakpointing, so breakpointing must
 * be handled in software.  The scheme is as follows:
 *
 * First, breakpoint code is downloaded to the DSP32C.  Then a goto is
 * planted in the user's code at the breakpoint address.  When execution
 * hits the planted goto and jumps to the breakpoint code, the DSP32C
 * registers are rolled out and the PC is notified that the breakpoint
 * has occurred.
 *
 * After notifying the PC of the break, the DSP32C goes into an infinite
 * loop waiting for a signal from the PC that is should continue on.  To
 * signal the DSP32C to restart, the PC pokes the breakpoint code with a
 * goto instruction returning to the user's code.  Basically, this is
 * self modifying code, and seems to be the only way to have breakpoints
 * without modifying the CAU and DAU flags.
 * 
 * Note that the breakpoint code trashes the value of register r21.  All
 * other CAU and DAU registers and flags are preserved.
 * 
 *
 * * THE BREAKPOINT CODE DSPMON.DSH FROM DSPMON.DSP IS INCLUDED
 *
 * The DSP32C code dspmon.dsp is assembled with the DSPASM /h option to
 * create a hexified code array dspmon.dsh that is included in this file.
 * Including the code as an array in dspmon.exe eliminates the need for
 * having an additional dspmon.dso file around that must be found each
 * time dspmon.exe is started.
 *
 *
 * * THE SYMBOL TABLE DSPMON.DSC FROM DSPMON.DSP IS INCLUDED
 *
 * The DSP32C code dspmon.dsp is also assembled with the DSPASM /sc
 * option to create a symbol table dspmon.dsc for inclusion in this
 * file.  Symbols which need to be known in both dspmon.dsp and dspmon.c
 * are automatically communicated to dspmon.c by this technique.  Note
 * that all dspmon.dsp symbols are automatically given a DSP_ prefix by
 * DSPASM in the C include file dspmon.dsc.
 *
 *
 * * DON'T OVERWRITE THE BREAKPOINT CODE ON THE DSP32C
 *
 * Currently, the DSP32C breakpoint code is downloaded to the internal
 * ram block RAM1 in the DSP32C high memory.  This memory is always present
 * at the same addresses for all DSP32C memory mode configurations,
 * and is probably the best choice for most users.  The base address to
 * download the breakpoint code to is given by the defined constant
 * BREAK_BASE in dspmon.dsp.
 *
 * When the breakpoint code is downloaded, it will overwrite any other
 * program or data there.  User applications should not have program or
 * data in ram block RAM1, and likewise should be careful not to
 * overwrite those areas if breakpointing is to work correctly.
 *
 *
 *
 */


/* Include the hexified code and symbol table from dspmon.dsp. */

#include "dspmon.dsh"
#include "dspmon.dsc"




/* DSP32C register stack. */

struct {

        /* This *MUST* agree with dspmon.dsp. */

        unsigned long cau[0x16];

          union { unsigned long l; float f; } dau[0x4];

             struct cau_flags { unsigned long n, z, v, c; } cau_flags;
             struct dau_flags { unsigned long n, z, v, u; } dau_flags;

        }

        regs;





/* DSP32C instructions. */

#define H_MASK          0x00FF0000L
#define L_MASK          0x0000FFFFL

#define R1_equ( n)      0xC0010000L | ((n&H_MASK)<<5) | (n&L_MASK)
#define Goto( a)        0xA0000000L | ((a&H_MASK)<<5) | (a&L_MASK)
#define Nop             0x00000000L

unsigned long

    users_inst[3],

        break_inst[3] = {  /* instructions to plant at the breakpoint */
        
                Nop,
                Goto( DSP_break),
                Nop

                },

          restart[2];





/* Current execution status and breakpoint address. */

#define HALTED          1       /* execution status values */
#define EXECUTING       2
#define BREAKPOINTING   3

  unsigned execution_status = HALTED; unsigned long breakpoint = 0;




void SetBreakpoint( void) {

        unsigned long signal;



        /* Save the user's instructions at the breakpoint. */
        
        DSP32C_Get( breakpoint, users_inst, sizeof( users_inst));



        /* Plant a breakpoint branch in the user's code. */

        DSP32C_Put( break_inst, breakpoint, sizeof( break_inst));



        /* Clear the DSP32C's break signal. */

        signal = 0;

            DSP32C_Put( &signal, DSP_signal, sizeof( signal));



}
void RestartFromBreakpoint( void) {


        /* Restart from the DSP32C's breakpoint code. */

        DSP32C_PutLong( Goto( DSP_break_restart), DSP_break_wait);



}
int WaitForBreak( void) {

        unsigned long signal; int status;




        /* Wait for the DSP32C's breakpoint signal or a kb interrupt. */

        while ( 1 ) {


            DSP32C_Get( DSP_signal, &signal, sizeof( signal));



                /* If the break occured, get the register dump. */

                if ( signal ) {
                        
                        DSP32C_Get( DSP_regs, &regs, sizeof( regs));

                          status = 1; break;

                        }



                /* Otherwise, check for a kb interrupt. */

                else if ( LL_KeyPress() )
                        
                            { DSP32C_Halt(); status = 0; break; }

            }





        /* Restore the user's instructions at the breakpoint. */

        DSP32C_Put( users_inst, breakpoint, sizeof( users_inst));



        /* Poke the DSP32C with restart code for the next breakpoint. */

        restart[0] = R1_equ( regs.cau[0]);
        restart[1] =   Goto( breakpoint);

          DSP32C_Put( restart, DSP_break_restart, sizeof( restart));



        /* Return whether the breakpoint occured or got a kbhit. */

        return( status);



}
int ExecuteBreakpoint( void) {

        unsigned status;
        


        /* If this is the first breakpoint, download and start. */
        
        if ( execution_status == HALTED ) {
                
                DSP32C_SrHexLoad( dspmon_dsh);
                
                    SetBreakpoint();

                    DSP32C_Go();

                }

        
        /* Otherwise restart from a previous breakpoint. */
        
        else /* execution_status == BREAKPOINTING */ {
                
                SetBreakpoint();

                    RestartFromBreakpoint();

                }



        /* Wait for the breakpoint. */
        
        return( WaitForBreak() );

}
















/* COMMAND LINE BUFFER AND POINTER:
 *
 * Responses the user types in at the command line are stored here.
 *
 *
 *
 */

#define MAXCOMMAND      50

char  command_buffer[MAXCOMMAND];
char *cbuf;
int   command_length = 0;









/* SHOW HELP COMMAND:
 *
 *
 *
 *
 */

char *HelpMsg[] = {

        "                                                           ",
        "  ** Commands **                                           ",
        "                                                           ",
        "   d[type] [addr] [nbytes]  display memory                 ",
        "   e[type] [addr]           enter memory value             ",
        "   c [addr] [nbytes]        clear memory                   ",
        "   u [addr] [nbytes]        unassemble memory              ",
        "                                                           ",
        "   l filename               load a dso, dss, or out file   ",
        "   i                        enable eni interrupt flag      ",
        "                                                           ",
        "   y [nstart] [nlines]      display symbol table           ",
        "   r                        display registers              ",
        "   ?                        display help text              ",
        "                                                           ",
        "   g [addr]                 go with optional breakpoint    ",
        "   s                        single step                    ",
        "   h                        halt and reset pc to 0         ",
        "   q                        quit                           ",
        "                                                           ",
        "    where type = (b)yte, (w)ord, (l)ong, dsp(f), (i)eee    ",
        "      addresses may be symbolic if dss file is loaded      ",
        "      addr and nbytes in hex, nstart and nlines in decimal ",
        "                                                           ",

        NULL,

        };


void ShowHelp( void) {

        int imsg;


        /* Show help text. */

        for ( imsg = 0 ; HelpMsg[imsg] ; )

                LL_PutS( HelpMsg[imsg++], 1);

}






/* SHOW SYMBOL COMMAND:
 *
 *
 *
 *
 */


struct symbol_data { 

        unsigned long start; 
        unsigned long nlines; 

        }

        s = { 0L, 23L };


void ShowSymbols( void) {

        int i, j, npage;

        SYMBOL *start, *sym; char buffer[SCREEN_WIDTH];




        /* Special case no symbols. */

        if ( nsymbols == 0 )

                { ShowError( "No symbol table loaded"); return; }



        /* Ensure user entered valid start value. */

        if (s.start >= nsymbols)
                s.start = 0;



        /* Display one page of the symbol table. */


        npage = s.nlines;
        start = &symbol_table[s.start];

        sym   = start;
        i     = 0;
        j     = s.start;


        LL_PutS( "Symbol table ", 1);

        while ( i < npage ) {

                if ( sym < symbol_table+nsymbols ) {

                        sprintf( buffer, "      %3d:  0x%08lX = %-31.31s",
                                                  j, sym->value, sym->name );

                        LL_PutS( buffer, 1);

                        sym++;

                        }

                i++; j++;

                }




        /* Advance default start to the end of symbols just displayed. */

        s.start += s.nlines;
        if (s.start >= nsymbols)
                s.start = 0;

        return;


}


/* DISPLAY MEMORY COMMAND:
 *
 *
 *
 *
 */

struct display_memory {

        int           type;
        unsigned long start;
        unsigned long nbytes;

        }

        d = { DSPASM_LONG, 0L, 0x5CL };




char *dprintf( int type, unsigned long addr) {

        int b; static char buf[80]; char *name;

        union {

                unsigned char  b[4];
                unsigned short w[2];
                unsigned long  l;
                float          f;
                float          i;

                } data;



        /* Read and format DSP32C data. */

#define PRINTABLE( c)   (isprint( c) ? c : '.')


        DSP32C_Get( addr, &data, sizeof( data));


        name = SymbolicName( addr, DSPASM_DATA);

          b = sprintf( &buf[0], "%12.12s %06lX: ", name, addr&MAXADDRESS);


        switch ( type ) {

            case DSPASM_BYTE:

                b += sprintf( &buf[b],

                        "%02X %02X %02X %02X      ASCII [%c%c%c%c]",

                                data.b[0], data.b[1], data.b[2], data.b[3],

                                        PRINTABLE( data.b[0]),
                                        PRINTABLE( data.b[1]),
                                        PRINTABLE( data.b[2]),
                                        PRINTABLE( data.b[3])
                                );
                break;


            case DSPASM_WORD:

                b += sprintf( &buf[b], "%04X %04X", data.w[0], data.w[1]);
                break;


            case DSPASM_LONG:

                b += sprintf( &buf[b], "%08lX", data.l);
                break;


            case DSPASM_DSPF:

                b += sprintf( &buf[b], "%s    DSPF", DSP32C_DspfToA( &data.f));
                break;


            case DSPASM_IEEE:

                b += sprintf( &buf[b], "%s    IEEE", DSP32C_IeeeToA( &data.i));
                break;

                }


        return( buf);


}
void Memory( void) {

        int ni, i; unsigned long addr;


        /* Display DSP32C memory. */

          ni = (d.nbytes+3)/4;
        addr =  d.start;


        LL_PutS( "Display memory", 1);

        for ( i = 0 ; i < ni ; i++ , addr += 4 )

                LL_PutS( dprintf( d.type, addr), 1);



        /* Move default start ahead to the end of memory just displayed. */

        d.start = addr;
        if (d.start > MAXADDRESS)
                d.start = 0;
}







/* ENTER MEMORY VALUE COMMAND:
 *
 *
 *
 */

struct enter_data { 

        int           type; 
        unsigned long addr; 

        } 

        e = { DSPASM_LONG, 0L };


void Enter( void) {

        unsigned long addr;

        char *type, msg[MAXCOMMAND]; unsigned n;


        union {

                unsigned short w;
                unsigned long  l;
                float          f;
                float          i;

                } value;






        /* Don't allow byte entry, the PIO is 16 bits wide ... */

        if ( e.type == DSPASM_BYTE ) {

                ShowError( "Byte entry not allowed, try word etc");
                
                return;

                }



        /* Let the user enter some new values. */

        switch ( e.type ) {

                case DSPASM_WORD: type = "WORD"; break;
                case DSPASM_LONG: type = "LONG"; break;
                case DSPASM_DSPF: type = "DSPF"; break;
                case DSPASM_IEEE: type = "IEEE"; break;

                }

        addr = e.addr;



  get_value:

        sprintf( msg, "Value (%s) for 0x%06lX ? ", type, addr&MAXADDRESS);

        CmdLineGets( msg); space();



             /* Poke the DSP32C with the value. */

             switch ( e.type ) {

                case DSPASM_WORD:

                        if ( hexnum( 4, &value.l) && eob() ) {

                            DSP32C_Put( &value.w, addr, sizeof( unsigned));

                                  addr += 2; break;
                                }

                        return;


                case DSPASM_LONG:

                        if ( hexnum( 8, &value.l) && eob() ) {

                            DSP32C_Put(&value.l, addr, sizeof( unsigned long));

                                  addr += 4; break;
                                }

                        return;


                case DSPASM_DSPF:

                        n = DSP32C_AToDspf( cbuf, &value.f);

                        if ( n ) {

                          cbuf += n; if ( eob() ) {

                            DSP32C_Put(&value.l, addr, sizeof( unsigned long));

                                  addr += 4; break;
                               }
                           }

                        return;


                case DSPASM_IEEE:

                        n = DSP32C_AToIeee( cbuf, &value.i);

                        if ( n ) {

                          cbuf += n; if ( eob() ) {

                            DSP32C_Put(&value.l, addr, sizeof( unsigned long));

                                  addr += 4; break;
                                }
                            }

                        return;

                }



                goto get_value;
}





/* CLEAR MEMORY COMMAND:
 *
 *
 *
 *
 */

struct clear_data { 

        unsigned long start;
        unsigned long nbytes;

        } 

        c = { 0L, 0x5CL };


void Clear( void) {

        unsigned long addr;

        long nwords;

        static unsigned long zero = 0L;



        /* Special case nbytes = 0. */

        if ( c.nbytes == 0 )

                return;



        /* Clear off memory, rounding nbytes up to words. */

          addr = c.start;
        nwords = (c.nbytes+1)/2;


        while ( nwords-- > 0 ) {


                /* Clear off a location. */

                DSP32C_Put( &zero, addr, sizeof( unsigned short)); addr += 2;



                /* Let the user break out with a keypress hit if needed. */

                if ( LL_KeyPress() ) {

                    ShowError( "Keyboard interrupt, clear not finished");

                         break;

                     }

                }

}





/* UNASSEMBLE (DISASSEMBLE) MEMORY COMMAND:
 *
 *
 *
 *
 *
 */

struct unassemble_data { 

        unsigned long start;
        unsigned long nbytes; 

        } 

        u = { 0L, 0x5CL };


char *uprintf( unsigned long addr) {

        unsigned long value;

        char inst[SCREEN_WIDTH], *name; static char buf[SCREEN_WIDTH];



        /* Disassemble a DSP32C memory location. */

        DSP32C_Get( addr, &value, sizeof( unsigned long));

        DSP32C_Decode( value, inst);

        name = SymbolicName( addr, DSPASM_PROGRAM);

        sprintf( buf, "%12.12s %06lX: %08lX  %-48.48s",

                                        name,
                                        addr & MAXADDRESS,
                                        value,
                                        inst

                                        );


        /* Put a highlight carot on the breakpoint address. */

        if ( addr == breakpoint )

               buf[0] = '>';


        return( buf);


}
void Unassemble( void) {

        int ni, i; unsigned long addr; 



        /* Display DSP32C memory as unassembled mnemonics. */


          ni = (u.nbytes+3)/4;
        addr = u.start;


        LL_PutS( "Unassemble memory", 1);

        for ( i = 0 ; i < ni ; i++ , addr += 4 )

                LL_PutS( uprintf( addr), 1);


        /* Move default start ahead to the end of memory just unassembled. */

        u.start = addr;
        if (u.start > MAXADDRESS)
                u.start = 0;

}





/* LOAD AN INPUT FILE COMMAND:
 *
 *
 *
 *
 */

void Load( char *filename) {


        /* Try to download a SR .dso object file.
         *
         * Note that SrLoad automatically does a halt when downloading,
         * and the breakpoint and execution status are updated to reflect
         * that.
         *
         *
         *
         */

        if ( DSP32C_SrLoad( filename, NO_REPORT) ) {

                      breakpoint = 0;
                execution_status = HALTED;

                ClearSymbolTable();

                         u.start = 0;

                }



        /* Try to load a SR .dss symbol table file. */

        else if ( ReadSymbols( filename) )

                /* downloaded a .dss symbol table file */ ;



        /* Try to download a WE AT&T .out coff object file. */

        else if ( DSP32C_WeLoad( filename, NO_REPORT) ) {

                breakpoint = 0; execution_status = HALTED;

                  ClearSymbolTable();

                    u.start = 0;
                }


        /* Unrecognized file format. */

        else {

                ShowError( "Unable to read input file");

                return;

                }

}



/* REGISTER DISPLAY COMMAND:
 *
 *
 *
 *
 *
 *
 */

char          /* flag and register display formats */

    *flag_fmt = "%-46s CAU(%c%c%c%c) DAU(%c%c%c%c) (0x%06lX)",

       *unknown_flag_fmt = "%-46s CAU(****) DAU(****) (0x%06lX)",


 *register_fmt[] = {

   "         r1 0x%06lX    r7 0x%06lX   r13 0x%06lX   r19 0x%06lX         ",
   "         r2 0x%06lX    r8 0x%06lX   r14 0x%06lX   r20 0x%06lX         ",
   "         r3 0x%06lX    r9 0x%06lX   r15 0x%06lX   r21 0x%06lX  (****) ",
   "         r4 0x%06lX   r10 0x%06lX   r16 0x%06lX   r22 0x%06lX  (ivtp) ",
   "         r5 0x%06lX   r11 0x%06lX   r17 0x%06lX                       ",
   "         r6 0x%06lX   r12 0x%06lX   r18 0x%06lX                       ",
   "                                                                      ",
   "      a0 %-13s (0x%08lX)      a2 %-13s (0x%08lX)    ",
   "      a1 %-13s (0x%08lX)      a3 %-13s (0x%08lX)    ",

   },

*unknown_register_fmt[] = {

 "         r1 0x******    r7 0x******   r13 0x******   r19 0x******         ",
 "         r2 0x******    r8 0x******   r14 0x******   r20 0x******         ",
 "         r3 0x******    r9 0x******   r15 0x******   r21 0x******  (****) ",
 "         r4 0x******   r10 0x******   r16 0x******   r22 0x******  (ivtp) ",
 "         r5 0x******   r11 0x******   r17 0x******                        ",
 "         r6 0x******   r12 0x******   r18 0x******                        ",
 "                                                                          ",
 "      a0 ************* (0x********)      a2 ************* (0x********)    ",
 "      a1 ************* (0x********)      a3 ************* (0x********)    ",

 };


char *rprintf( int ifmt) {

        static char buf[2*SCREEN_WIDTH]; /* 2* for sprintf buf growth */

            char num0[32], num1[32], num2[32], num3[32];



        /* Format a line of the DSP32C register display. */

        if ( 0 <= ifmt && ifmt <= 3 ) {

                sprintf( buf, register_fmt[ifmt],

                        regs.cau[ifmt+ 0] & MAXADDRESS,
                        regs.cau[ifmt+ 6] & MAXADDRESS,
                        regs.cau[ifmt+12] & MAXADDRESS,
                        regs.cau[ifmt+18] & MAXADDRESS

                        );
                }

        else if ( 4 <= ifmt && ifmt <= 5 ) {

                sprintf( buf, register_fmt[ifmt],

                        regs.cau[ifmt+ 0] & MAXADDRESS,
                        regs.cau[ifmt+ 6] & MAXADDRESS,
                        regs.cau[ifmt+12] & MAXADDRESS

                        );
                }

        else if ( ifmt == 7) {

                strcpy( num0, DSP32C_DspfToA( &regs.dau[0].f));
                strcpy( num2, DSP32C_DspfToA( &regs.dau[2].f));

                sprintf( buf, register_fmt[ifmt],

                        num0, regs.dau[0].l,
                        num2, regs.dau[2].l

                        );
                }

        else if ( ifmt == 8 ) {

                strcpy( num1, DSP32C_DspfToA( &regs.dau[1].f));
                strcpy( num3, DSP32C_DspfToA( &regs.dau[3].f));

                  sprintf( buf, register_fmt[ifmt],

                        num1, regs.dau[1].l,
                        num3, regs.dau[3].l

                        );

                }

        else
                sprintf( buf, register_fmt[ifmt]);


        return( buf);


}
void Registers( void) {

        int ifmt;

        char buffer[SCREEN_WIDTH];

        char *register_title = "Registers";


        /* Display the window title bar, flags and breakpoint. */


        if ( execution_status == BREAKPOINTING ) {

                    sprintf( buffer, flag_fmt,

                                register_title,

                                regs.cau_flags.n ? 'N' : 'n' ,
                                regs.cau_flags.z ? 'Z' : 'z' ,
                                regs.cau_flags.v ? 'V' : 'v' ,
                                regs.cau_flags.c ? 'C' : 'c' ,

                                regs.dau_flags.n ? 'N' : 'n' ,
                                regs.dau_flags.z ? 'Z' : 'z' ,
                                regs.dau_flags.v ? 'V' : 'v' ,
                                regs.dau_flags.u ? 'U' : 'u' ,

                                breakpoint & MAXADDRESS

                                );
                }

        else
                sprintf( buffer, unknown_flag_fmt,

                                  register_title, breakpoint&MAXADDRESS );


        LL_PutS( buffer, 1);




        /* Display the DSP32C registers. */


        if ( execution_status == BREAKPOINTING ) {

                for ( ifmt = 0 ; ifmt < 9 ; ifmt++ )

                        LL_PutS( rprintf( ifmt), 1);

                }

        else {

                /* Display the unknown register display. */

                for ( ifmt = 0 ; ifmt < 9 ; ifmt++ )

                    LL_PutS( unknown_register_fmt[ifmt], 1);

                }


        /* Plus unassemble the current instruction or show executing. */

        LL_PutS( "", 1);

        if ( execution_status == BREAKPOINTING )

                LL_PutS( uprintf( breakpoint & MAXADDRESS), 1);


        else if ( execution_status == EXECUTING )

                LL_PutS( "                     Executing ...", 1);

}






/* GENERAL COMMANDS:
 *
 *
 *
 *
 *
 */

void update_breakpoint( void) {

        int hscroll;



        /* Scroll the unassemble window to show the breakpoint. */

        hscroll = (u.nbytes+3)/4;


        if ( breakpoint < u.start )

                u.start = breakpoint;


        else if ( breakpoint >= u.start + 4*(hscroll/2) )

                u.start = breakpoint - 4*(hscroll/2);

}
void kb_interrupt( void) {


        /* Halt if execution was broken with a kb interrupt. */
        
        DSP32C_Halt();
        
            breakpoint = 0; execution_status = HALTED;


        /* Give the user an error message. */

        ShowError( "Execution interrupted with kbhit, DSP32C halted" );



        /* Update the screen. */
                
        update_breakpoint();


}
void CommandGo( void) {


        /* If already executing, tell the user they should stop first. */
        
        if ( execution_status == EXECUTING )

                ShowError( "Already executing, halt first");



        /* Else if halted start continuous execution from reset. */
        
        else if ( execution_status == HALTED ) {
                

                DSP32C_Go();

                  ShowMsg( "Executing from reset until halted");


                execution_status = EXECUTING;

                }



        /* Else restart continuous execution from a previous breakpoint. */

        else { /* execution_status == BREAKPOINTING */
                

                RestartFromBreakpoint();

                  ShowMsg( "Continuing execution until halted");


                execution_status = EXECUTING;

                }

}
void CommandBreakpoint( unsigned long newbreak) {


        /* If already executing, tell the user they should stop first. */
        
        if ( execution_status == EXECUTING ) {

                ShowError( "Already executing, halt first");
                }


        /* Otherwise set the breakpoint and execute. */

        else { /* execution_status == HALTED or BREAKPOINTING */


            breakpoint = newbreak;


                if ( ExecuteBreakpoint() )

                        execution_status = BREAKPOINTING;
                        
                else
                        kb_interrupt();

            }
   

        /* Update the screen. */
    
        update_breakpoint();


}
void CommandStep( void) {

        unsigned long newbreak;


        /* Move the breakpoint one instruction onward and execute. */

        newbreak = breakpoint + 4;
        
                CommandBreakpoint( newbreak);


        /* Display unassembled current instruction. */

        LL_PutS( uprintf( breakpoint & MAXADDRESS), 1);

}
void CommandHalt( void) {


        /* Halt the DSP32C, resetting to 0. */

        DSP32C_Halt();
        
            breakpoint = 0; execution_status = HALTED;



        /* Give the user a message. */

        ShowMsg( "DSP32C halted");



        /* Update the screen. */

        update_breakpoint();



}
void CommandEnableEni( void) {


        /* Enable the PIR interrupt. */

        DSP32C_SetEni( 1);


}
void CommandQuit( void) {


        /* Quit. */

        exit( 0);



}












/* DSPMON COMMAND LINE PARSING:
 *
 * These functions parse arguments specified on the DSPMON command line.
 *
 *
 *
 */

int space( void) {

        char *save;


        /* Skip over any white space in the command buffer. */

        save = cbuf;

          if ( isspace( *cbuf) )

              { while ( isspace( *cbuf) ) cbuf++; return( 1); }


          else
                { cbuf = save; return( 0); }


}
int type( int *flag) {

        struct type { int c, type_flag; } *t;

            static struct type type_table[] = {

                        { 'b',  DSPASM_BYTE, },
                        { 'w',  DSPASM_WORD, },
                        { 'l',  DSPASM_LONG, },
                        { 'f',  DSPASM_DSPF, },
                        { 'i',  DSPASM_IEEE, },

                        {   0,     0, },

                        };


        /* Check for a legal type specifier. */

        for ( t = type_table ; t->c ; t++ ) {

                if ( *cbuf == t->c )

                        { cbuf++; *flag = t->type_flag; return( 1); }

                }


        return( 0);


}
int hexnum( int ndigits, unsigned long *value) {

        char *save; unsigned long num; int d, n;


        /* Check for an n digit hex number. */

        save = cbuf;

          num = 0L; n = 0;



          /* Skip any number of leading 0's. */

          while ( *cbuf == '0' )

                  cbuf++;


          /* Scan for up to ndigits significant hex digits. */

          while ( isxdigit( *cbuf) && n < ndigits ) {

                d = *cbuf++; num <<= 4;


                  if ( '0' <= d && d <= '9' )

                        num += (d - '0');

                  else if ( 'a' <= d && d <= 'f' )

                        num += (d - 'a' + 0xA);

                  else if ( 'A' <= d && d <= 'F' )

                        num += (d - 'A' + 0xA);

                 n++;

                }


        /* Make sure something was read in. */

        if ( cbuf > save && n <= ndigits )

                { *value = num; return( 1); }

        else

                { cbuf = save; return( 0); }

}
int decnum( int ndigits, unsigned long *value) {

        char *save; unsigned long num; int d, n;


        /* Check for an n digit decimal number. */

        save = cbuf;

          num = 0L; n = 0;



          /* Skip any number of leading 0's. */

          while ( *cbuf == '0' )

                  cbuf++;


          /* Scan for up to ndigits significant digits. */

          while ( isdigit( *cbuf) && n < ndigits ) {

                d = *cbuf++; num *= 10;


                  if ( '0' <= d && d <= '9' )

                        num += (d - '0');

                 n++;

                }


        /* Make sure something was read in. */

        if ( cbuf > save && n <= ndigits )

                { *value = num; return( 1); }

        else

                { cbuf = save; return( 0); }

}
int address( unsigned long *value) {

        char *save;

            unsigned long addr; int d, ndigits;

                char name_buffer[MAXCOMMAND], *name; SYMBOL *s; int type;



        /* Check for a symbolic address, requiring leading space. */

        save = cbuf;

        if ( space() && (isalpha( *cbuf) || *cbuf == '_') ) {

            name = name_buffer;

                while ( isalnum( *cbuf) || *cbuf == '_' )

                        *name++ = *cbuf++;

                *name = '\0';



                /* Try to find the name on the symbol table. */

                if ( (s = SymbolicValue( name_buffer, &type)) != NULL )

                        { *value = s->value; return( 1); }


                }



        /* Check for a 6 digit hex address, again requiring leading space. */

        cbuf = save;

        if ( space() && hexnum( 6, &addr) ) {

                /* Make sure 24 bits, and round to a word location. */

                  if ( addr <= MAXADDRESS )

                    { addr &= 0xFFFFFFFEL; *value = addr; return( 1); }

                }



        /* Didn't find absolute or symbolic, restore cbuf and return. */

        cbuf = save;

            return( 0);


}
int nbytes( unsigned long *value) {

        char *save;


        /* Check for an 8 digit hex number, requiring leading space. */

        save = cbuf;

          if ( space() && hexnum( 8, value) )

              return( 1);

          else
                { cbuf = save; return( 0); }


}
int ndecimal( unsigned long *value) {

        char *save;


        /* Check for an 3 digit decimal number, requiring leading space. */

        save = cbuf;

          if ( space() && decnum( 3, value) )

              return( 1);

          else
                { cbuf = save; return( 0); }


}
char *filename( void) {

        char *save, *p;

            static char buf[MAXCOMMAND];


        /* Require a leading space. */

        save = cbuf;

        if ( !space() )

                { cbuf = save; return( 0); }


        /* Check for a filename. */

        p = buf;

            while ( isprint( *cbuf) ) *p++ = *cbuf++;


          *p = '\0';


        /* Make sure something was read in. */

        if ( p > buf )

                return( buf);

        else
                { cbuf = save; return( NULL); }


}
int eob( void) {


        /* Check for the end of the command buffer. */

        space();

          return( *cbuf == '\0' ? 1 : 0 );

}
int eoc( void) {


        /* Check for the end of the command buffer, with an error message. */

        space();


        if ( *cbuf == '\0' ) {

                return( 1);

                }

        else {

                ShowError( "Illegal command");

                return( 0);

                }
}








/* DSPMON COMMAND LINE:
 *
 * The following functions handle the DSPMON command line.
 *
 *
 *
 *
 *
 */

void CmdLineClear( void) {


        /* Clear the command line buffer. */

        cbuf = command_buffer; command_length = 0;

}
void CmdLineGets( char *msg) {


        /* Display a message string and get a command. */

        LL_PutS( msg, 0);


        CmdLineClear();


        command_length = StrGet( cbuf, 80 );
        cbuf           = command_buffer;

}




#define UNASSEMBLE              1
#define DISPLAY_MEMORY          2
#define REGISTERS               3
#define SINGLE_STEP             4
#define GO                      5
#define HALT                    6
#define ENTER_MEMORY            7
#define CLEAR_MEMORY            8
#define ENABLE_ENI              9
#define LOAD                    10
#define SYMBOLS                 11
#define HELP                    12
#define QUIT                    13

struct

  command { char *name; int flag; } command_table[] = {

        "c",            CLEAR_MEMORY,
        "d",            DISPLAY_MEMORY,
        "e",            ENTER_MEMORY,
        "g",            GO,
        "h",            HALT,
        "i",            ENABLE_ENI,
        "l",            LOAD,
        "quit",         QUIT,
        "q",            QUIT,
        "r",            REGISTERS,
        "s",            SINGLE_STEP,
        "u",            UNASSEMBLE,
        "y",            SYMBOLS,
        "?",            HELP,

        NULL,           0,

        };



void CmdLineProcessor( void) {

        struct command *cmd; char *p;

            unsigned long newbreak;



        /* Process the entered command. */


            CmdLineGets( "[cdeghilqrsuy?] " );


            for ( cmd = command_table ; cmd->name ; cmd++ ) {

                if ( StrScan( cbuf, cmd->name) )

                        { cbuf += strlen( cmd->name); break; }

                }


        /* Parse the rest of the command, and take the action. */

        switch ( cmd->flag ) {

                case DISPLAY_MEMORY:

                        type( &d.type); address( &d.start); nbytes( &d.nbytes);

                        if ( eoc() )

                               Memory();

                        break;

                case ENTER_MEMORY:

                        type( &e.type); address( &e.addr);

                        if ( eoc() )

                                Enter();
                        break;

                case CLEAR_MEMORY:

                        address( &c.start); nbytes( &c.nbytes);

                        if ( eoc() )

                                Clear();
                        break;

                case UNASSEMBLE:

                        address( &u.start); nbytes( &u.nbytes);

                        if ( eoc() )

                                 Unassemble();
                        break;

                case REGISTERS:

                        if ( eoc() )

                                 Registers();
                        break;

                case LOAD:

                        p = filename();

                        if ( eoc() )

                              Load( p);

                        break;

                case GO:

                        if ( address( &newbreak) && eoc() ) {
                                
                                newbreak &= 0xFFFFFFFCL; /* round to 4 byte */

                                    CommandBreakpoint( newbreak);

                                }

                        else if ( eoc() )

                                CommandGo();

                        break;

                case SINGLE_STEP:

                        if ( eoc() )

                                CommandStep();

                        break;

                case HALT:

                        if ( eoc() )

                                CommandHalt();
                        break;

                case ENABLE_ENI:

                        if ( eoc() )

                                CommandEnableEni();
                        break;

                case HELP:

                        if ( eoc() )

                                ShowHelp();

                        break;

                case SYMBOLS:

                        ndecimal( &s.start); ndecimal( &s.nlines);

                        if ( eoc() )

                                ShowSymbols();

                        break;

                case QUIT:

                        if ( eoc() )

                                CommandQuit();
                        break;

                default:

                        ShowError( "Unknown or illegal command");

                        break;

                }



        /* Clear the command buffer. */
        
        CmdLineClear();


}











/* MAIN:
 *
 * Get DOS command line arguments and start the DSPMON command processor.
 *
 *
 *
 *
 */

#define NO_CPU                  -1
#define MAXFILES                 2


int main( int argc, char **argv) {

        int nfields, ncpu, nbytes;

        char *filename[MAXFILES];

            int ifile, nfiles, dso, dss, out;


#ifdef SROS_LINUX

        /* Set Linux non-blocking reads to allow for keyboard interrupts. */

        if ( SetNonBlock( 1 ) == -1)
                ShowMsg( "No keyboard interrupts allowed");
#endif



        /* Process the command line. */

          ncpu = NO_CPU;
         ifile = 0;
        nfiles = 0;


        while ( --argc > 0 ) { ++argv;



                /* Check for the /ncpu DSPMUL option. */

                if ( **argv == '/' ) {


                        nfields = sscanf( *argv, "/%d%n", &ncpu, &nbytes);


                        if ( nfields != 1 || (*argv)[nbytes] != '\0' )

                                DOS_Error( "Illegal command line option ...");


                        else if ( ncpu < 0 || 3 < ncpu )

                                DOS_Error( "Illegal DSPMUL cpu number ...");

                        }



                /* If not an option, then assume argv is an input filename. */

                else {

                        filename[ifile++] = *argv;


                            if ( ++nfiles > MAXFILES )

                                DOS_Error( "Too many input files ...");
                        }

                }







        /* Initialize the DSP32C board. */

        if ( ncpu > NO_CPU )

                DSP32C_SelectCpu( ncpu);  /** for DSPMUL boards ... **/


        if ( !DSP32C_Init() )

                DOS_Warning( "Unable to intialize DSP32C board");






        /* Process any input files. */

        dso = 0;
        dss = 0;
        out = 0;

        for ( ifile = 0 ; ifile < nfiles ; ifile++ ) {


                if ( DSP32C_SrLoad( filename[ifile], NO_REPORT) )

                        dso++;

                else if ( ReadSymbols( filename[ifile]) )

                        dss++;

                else if ( DSP32C_WeLoad( filename[ifile], NO_REPORT) )

                        out++;


                else
                      DOS_Error( "Illegal input file ...");


                }


        if ( dso > 1 || out > 1 || (dso > 0 && out > 0) )

                DOS_Error( "Too many object files on command line ...");

        else if ( dss > 1 )

                DOS_Error( "Too many symbol table files on command line ...");






        /* Start with help screen. */

        ShowHelp();




        /* Start the keyboard command processor. */

 nextcmd:
        CmdLineProcessor();
        goto nextcmd;



        exit( 0);
}

