/* FILE: dspa64.c               Copyright (c), Symmetric Research, 1997-2000
 *
 * SR DSPA64 low level/DLL interface library.
 *
 *
 * NOTES:
 *
 * These functions work in conjunction the DSP code contained in the
 * file dspa64.dsp and the 32 bit DSPIO DSP32C tools.
 *
 *
 *
 *
 *
 */

#include <stdio.h>
#include <math.h>






/* INCLUDE DSPIO.H AND DSPA64.H:
 *
 * Note that for building the library, the dspio.h and dspa64.h 
 * _DLLEXPORT or _STATIC should be defined from the compile command line.  
 *
 * See the makefile and include files for details.
 *
 *
 */

#include "dspio.h"
#include "dspa64.h"






/* RELEASE REV INFO:
 *
 * Allow the user to determine the library rev they are working with.  See
 * dspa64.h for more details.
 *
 *
 */

VARTYPE( INT) DSPA64_Rev = DSPA64_REV;

FUNCTYPE( INT) DSPA64_GetRev( void) {

        /* Return the library rev number. */

        return( DSPA64_Rev);
}






/* KEYBOARD HELPER FUNCTIONS:

These are needed to help smooth over keyboard behavior differences
between Linux and Windows.

GetKey has two behaviors.  If wait is true, it will wait for getch to
return a character.  Otherwise, it will use kbhit to check for an available
character.  If one is there, getch reads and returns it.  Otherwise, EOF
is returned.

GetKeyWait and GetKeyCheck are just handy shortcut variations.

*/

FUNCTYPE( INT) GetKey( int wait ) {

        int c;

        /* Get a ready keypress or wait until one is ready if requested. */

#ifdef SROS_LINUX

        c = getchar();
        while ( (c == EOF) && wait )
                c = getchar();
        
#else

        c = EOF;
        if ( kbhit() || wait ) {
                c = getch();
                if (c == 0xD) printf("\n"); /* add newline if char was ENTER */
                }
#endif

        return( c );
}

FUNCTYPE( INT) GetKeyCheck( void ) { return( GetKey( KEY_CHECK ) ); }
FUNCTYPE( INT) GetKeyWait(  void ) { return( GetKey( KEY_WAIT  ) ); }







/* DSP32C/DSPA64 SYSTEM PARAMETERS:
 *
 * This data is downloaded to the DSP32C board, defining the system
 * configuration for dspa64.dsp.
 *
 * The layout of the DSPA64 system parameters data structure here MUST
 * agree with that on the DSP32C board.  See dspa64.dsp for more details
 * on the DSP32C side.
 *
 * Note that this structure is NOT the same as the DSPA64_PARAMETERS
 * structure passed to DSPA64_Init().  It contains additional computed
 * information required by dspa64.dsp.
 *
 *
 *
 *
 *
 */

#include "dspa64.dsc"
#include "dspa64.dsh"


#define uDSP_NOT_READY  (unsigned)DSP_NOT_READY
#define C(N)            (DSP_C00 + (N)*sizeof(struct config_list_entry))
#define SKEW            3


struct DSPA64_system_parameters {

             float filter_scale;     /* *MUST* be float for DSP32C */
             float filter_coeff;

          unsigned long ncount;

          unsigned long buffer1;
          unsigned long buffer1_end;
          unsigned long buffer2;
          unsigned long buffer2_end;


         struct config_list_entry

             { unsigned long channel, filter, count, next; }

                config_list[ MAXCHANNELS];


        } SYSTEM;


        




/* DSPA64 EXECUTION CONTROL FUNCTIONS:
 *
 * Use these functions to initialize, start execution, and get data from
 * the DSP32C and DSPA64.
 *
 *
 *
 *
 */

FUNCTYPE( VOID) DSPA64_SetBaseIo( unsigned value) {

        DSP32C_SetBaseIo( value);

}
FUNCTYPE( INT) DSPA64_GetBaseIo( void) {

        return( DSP32C_GetBaseIo());

}


/* These two just exist to provide backwards compatibility for 3rd party apps. */

FUNCTYPE( VOID) DSPA64_SetBaseio( unsigned value) { DSPA64_SetBaseIo( value);}
FUNCTYPE( INT)  DSPA64_GetBaseio( void)           { return( DSPA64_GetBaseIo());}






FUNCTYPE( INT) DSPA64_Init(

          int nchannel,
          int nsample,
          int ncount,

        float filter_coeff,
        float filter_scale,

         int *scan_list

        )


{ int i, iskew; unsigned long nbuffer;



        /* Initialize the DSP32C board:
         *
         * Call this function to initialize the DSPA64 system parameters,
         * and download the DSP32C board.
         *
         *
         *
         */


        nbuffer = (long)nsample * nchannel * sizeof( short );


        SYSTEM.filter_scale  = filter_scale;
        SYSTEM.filter_coeff  = filter_coeff;

        SYSTEM.ncount        = ncount;

        SYSTEM.buffer1       = DSP_VALUES;
        SYSTEM.buffer1_end   = DSP_VALUES + 1*nbuffer;

        SYSTEM.buffer2       = DSP_VALUES + 1*nbuffer;
        SYSTEM.buffer2_end   = DSP_VALUES + 2*nbuffer;




        for ( i = 0 ; i < nchannel-1 ; i++ ) {

                iskew = (i+SKEW)%nchannel;

                SYSTEM.config_list[i].channel = scan_list[iskew];
                SYSTEM.config_list[i].filter  = 0;
                SYSTEM.config_list[i].count   = 0;
                SYSTEM.config_list[i].next    = C( i+1);

                }

        iskew = (i+SKEW)%nchannel;

        SYSTEM.config_list[i].channel = scan_list[iskew];
        SYSTEM.config_list[i].filter  = 0;
        SYSTEM.config_list[i].count   = 0;
        SYSTEM.config_list[i].next    = C( 0);  /* link back to channel 0 */

        i++;

        while ( i < MAXCHANNELS ) {

                /* zero off remaining config_list entries ... */

                SYSTEM.config_list[i].channel = 0;
                SYSTEM.config_list[i].filter  = 0;
                SYSTEM.config_list[i].count   = 0;
                SYSTEM.config_list[i].next    = 0;

                i++;

                }





        /* Initialize the DSP32C, download program and parameters. */

        if ( !DSP32C_Init() || !DSP32C_SrHexLoad( dspa64_dsh) )

                return( 0 );


        DSP32C_Put( &SYSTEM, DSP_SYSTEM, sizeof(SYSTEM));

        return( 1 );


}
FUNCTYPE( VOID) DSPA64_Start( void ) {


        /* Start DSP32C execution. */

        DSP32C_Go();

        DSP32C_SetPir( uDSP_NOT_READY);


}
FUNCTYPE( VOID) DSPA64_Stop( void ) {


        /* Stop DSP32C execution. */

        DSP32C_SetPir( uDSP_NOT_READY);

        DSP32C_Halt();


}
FUNCTYPE( VOID) DSPA64_ClearReady( void ) {


        /* Clear the DSP32C pir flag. */

        DSP32C_SetPir( uDSP_NOT_READY);


}
FUNCTYPE( INT) DSPA64_IsReady( void ) {

        int buffer;



        /* Call this function to poll for whether a DSP32C buffer has
         * become ready.
         *
         * If it has, a non zero value will be returned indicating which
         * one of the two dspa64.dsp ping pong buffers has become ready.
         *
         * Use the return value with DSPA64_get_data to get the right
         * buffer.
         *
         *
         */

        buffer = DSP32C_Pir();    /* poll the pir register */



        if ( buffer == DSP_NOT_READY )

                return( 0 );

        else
                return( buffer );


}
FUNCTYPE( INT) DSPA64_GetData( int status, short *pcbuffer, int nbytes ) {


        /* Read the buffer that has become ready back from the DSP32C.
         *
         *   Status should be the value returned by DSPA64_is_ready().
         *
         *
         *
         */

        switch ( status ) {


                case DSP_BUFFER1_READY:

                        DSP32C_Get( SYSTEM.buffer1, pcbuffer, nbytes);

                        DSP32C_SetPir( uDSP_NOT_READY);

                        return( 1 );


                case DSP_BUFFER2_READY:

                        DSP32C_Get( SYSTEM.buffer2, pcbuffer, nbytes);

                        DSP32C_SetPir( uDSP_NOT_READY);

                        return( 1 );


                default:

                        return( 0 );

                }

}







/* CONVERSION HELPER FUNCTIONS:
 *
 * These functions convert between the DSPA64 parameters and alternate
 * parameters the user may typically want to specify.
 *
 * The DSPA64_ntimingticks function should be used to set the global
 * ntimingticks variable that represents the number of clock ticks
 * allowed for conversion.  On the DSPA64 board, this number is 
 * controlled by the setting of JP60, the timing jumper.  The global 
 * variable is initialized to match the default factory setting.
 * After changing this variable, be sure to recompute sps and 
 * maximum rate.  This function can also be used to retrieve the
 * current setting of this variable by passing in an invalid newticks
 * value such as 0.
 *
 * The DSPA64_maximumrate function should be used to compute the 
 * maximum aggregate sampling rate, ie the total rate over all 
 * channels.  To get the maximum per channel rate, just divide by 
 * the number of channels.
 *
 * The DSPA64_buffersize function should be used to compute the size 
 * of a PC buffer to allocate.
 *
 * DSPA64_sps_to_ncount and DSPA64_ncount_to_sps convert between the
 * decimation count and samples per second on each channel.
 *
 * The filter parameter functions convert between the value of the
 * filter tail and filter coefficient and scale.
 *
 * Use these functions to help set up the DSPA64_parameters structure
 * used with DSPA64_init.
 *
 *
 *
 *
 */
                
int ntimingticks = MIN_NTICKS;  



FUNCTYPE( VOID) DSPA64_NTimingTicks( int newticks, int *nticks ) {


        if ((newticks >= MIN_NTICKS) && (newticks <= MAX_NTICKS))

                ntimingticks = newticks;


        if (nticks != NULL)

                *nticks = ntimingticks;


}
FUNCTYPE( VOID) DSPA64_MaximumRate( float *maxrate ) {

                
        /* Compute the maximum aggregate sampling rate. */

        if (ntimingticks <= 0)

                *maxrate = (float)0.0;

        else
                *maxrate = XTAL / (float)ntimingticks;
        

}
FUNCTYPE( VOID) DSPA64_BufferSize( int nchannel, int nsample, int *nbytes ) {

        
        /* Compute the number of bytes in an acquisition buffer. */

        *nbytes = nchannel * nsample * sizeof( short int);


}
FUNCTYPE( VOID) DSPA64_SpsToNCount( int nchannel, float sps, int *ncount ) {

        int nc1, nc2; float diff1, diff2, sps1, sps2;



        /* Convert a sampling rate in samples/s to the decimation count. */

        if ( nchannel <= 0 || sps <= 0.0 || ntimingticks <= 0 )

                { *ncount = 0; return; }



        nc1 = ( (int)( XTAL/(ntimingticks * nchannel * sps)) ) - 1;
        nc2 = nc1 + 1;

        DSPA64_NCountToSps( nchannel, nc1, &sps1 );
        DSPA64_NCountToSps( nchannel, nc2, &sps2 );

        diff1 = sps1 - sps;
        diff2 = sps  - sps2;


        *ncount = ( diff1 < diff2 ? nc1 : nc2 );


        if ( *ncount < 0 )

                *ncount = 0;


}
FUNCTYPE( VOID) DSPA64_NCountToSps( int nchannel, int ncount, float *sps ) {


        /* Convert a decimation count to the sampling rate in sps. */

        if ( nchannel <= 0 || ncount < 0 || ntimingticks <= 0)

                *sps = (float)0.0;

        else
                *sps = (float)( XTAL/((long)ntimingticks*nchannel*(ncount+1)) );



}
FUNCTYPE( VOID) DSPA64_TailToCoeff( int ncount, float tail, float *coeff ) {


        /* Given a specified filter tail, compute the filter coefficient.
         *
         * The relationship between the filter tail and filter
         * coefficient is:
         *
         *      tail = coeff ^ (ncount+1)
         *
         *
         *
         */

        if ( ncount < 0 )

                *coeff = (float) 0.0;

        else
                *coeff = (float)pow( tail, 1.0/(ncount+1) );



}
FUNCTYPE( VOID) DSPA64_CoeffToTail( int ncount, float coeff, float *tail ) {


        /* Compute the filter tail value from the filter coeff. */

        if ( (ncount < 0) || coeff <= 0.0 )

                { *tail = (float) 0.0; return; }

        else
                *tail = (float)pow( coeff, ncount+1 );



}
FUNCTYPE( VOID) DSPA64_NormalizeScale( float coeff, float *scale ) {


        /* Given the filter coefficient, compute a normalized scale. */

        *scale = (float)1.0 - coeff;


}

FUNCTYPE( INT) DSPA64_Open( void ) {

        /* Open DSP32C device. */

        return( DSP32C_Open() );
}

FUNCTYPE( INT) DSPA64_Close( void ) {

        /* Close DSP32C device. */

        return( DSP32C_Close() );
}
